#!/usr/bin/env python3
# -*- coding: utf-8 -*

import os
import sys


try:
    from aeadtvgen import aeadtvgen
except ImportError:
    print("================================================================")
    print("Warning: aeadtvgen not installed!!!! Use source package instead.")
    print("================================================================")
    sys.path.append(os.path.abspath('..'))
    from aeadtvgen import aeadtvgen

if __name__ == '__main__':
    # Path to library folder
    path = os.path.realpath('../../lib')
    # Test vectors message output format (Add len segment)
    msg_format = '--msg_format npub ad data len tag'.split()
    # Len segment format (Note: left shift by 3-bit so it represents bit size)
    len_format = '--len_format 61 64'.split() 
    # Specify destination folder for output files
    dest = '--dest ../../../hardware/dummy2/scripts'.split()

    # Run routine (add at least one to args array)
    gen_test_routine = '--gen_test_routine 1 20 2'.split()
    gen_random = '--gen_random 3'.split()        
    gen_custom = ['--gen_custom',   # Note: Use this format!!    
        '''\
        True,   False,      0,          20:
        0,      1,          100,        500:
        1,      1,          40,         100:
        ''']
    gen_single = ['--gen_single',
        '1',                                    # Encrypt/decrypt
        '000102030405060708090A0B0C0D0E0F',     #Key
        '000102030405060708090A0B0C0D0E0F',     #Npub
        '000102030405060708090A0B0C0D0E0F',     #Nsec (Ignored: npub_size=0)
        '000102030405060708090A0B0C0D0E0F',     #AD
        '000102030405060708090A0B0C0D0E0F',     #PT
        ]

    # Create argument options for aeadtvgen
    args = [
        path,                       # Library path
        'dummy2',                   # Library name
        '--npub_size', '128',       # Npub size
        '--max_io_per_line', '8',   # Limit data per line to 256-bit for readability
        '--block_size',    '128',   # Data block size
        '--block_size_ad', '96',    # AD block size
        '--human_readable',         # Generate a human readable text file
        '--verify_lib',             # Verify reference enc/dec in reference code
    ]      
    # add previously declared options
    args += msg_format + len_format
    args += dest + gen_test_routine   

    # Pass help flag through
    try:
        if (sys.argv[1] == '-h'):
            args.append('-h')
    except:
        pass

    # Call program
    aeadtvgen(args)
